unit photozou_api;
(* PhotozouAPI (http://photozou.jp) written by mineaki sakatoku@unoh 
 *
 *)
interface

uses
  Windows,SysUtils, Classes, Forms, IdHTTP, IdComponent, IdLogDebug, mmsystem;

const
  //todo: API_COMMAND_XXX
  API_COMMAND_NOP             = 'nop';
  API_COMMAND_PHOTO_ALBUM     = 'photo_album';
  API_COMMAND_USER_GROUP      = 'user_group';
  API_COMMAND_PHOTO_ADD       = 'photo_add';
  API_COMMAND_PHOTO_ADD_TAG   = 'photo_add_tag';
  API_COMMAND_PHOTO_COMMENT   = 'photo_comment';
  API_COMMAND_PHOTO_LIST      = 'photo_list';
  API_COMMAND_PHOTO_ADD_ALBUM = 'photo_add_album';

type
  // multi part form
  PPMultiFormItem = ^TPMultiFormItem;
  TPMultiFormItem = record
    key: UTF8String;
    val: UTF8String;
  end;

  TPMultiForm = class(TList)
  private
    FBoundary: UTF8String;
    function getBoundary: UTF8String;
  public
    procedure AddItem(key, val: UTF8String);
    procedure AddFileItem(key, filename, val: UTF8String; ctype: UTF8String = ''); // ctype͋łOK
    procedure Clear; override;
    function GetContetType: UTF8String;
    function GetPostData: UTF8String;
    property Boundary: UTF8String read getBoundary write FBoundary;
  end;

  TPhotoType = (ptNone, ptPublic, ptEveryone, ptAlbum, ptComment,
                ptFriend, ptPopular, ptCC);
  TPhotoLicense = (none, by, by_sa, by_nd, by_nc, by_nc_sa, by_nc_nd);

  TPermType  = (allow, deny);
  TPermType2 = (net, everyone, all, user_group);

  // api
  TPhotozouApi = class(TComponent)
  private
    FUserAddress  : string;
    FUserPassword : string;
    FUserAgent    : string;
    FUserId: Integer;
    FPhotoId: Integer;
    FAPIUrl: string;
    FHttp:  TIdHTTP;
    FOnWork: TWorkEvent;
    FOnWorkBegin: TWorkBeginEvent;
    FOnWorkEnd: TWorkEndEvent;
    FErrorMessage: string;
    FUseProxy: Boolean;
    FLogger: TIdLogDebug;
    FLastTime: DWORD;
    procedure SetOnWork(const Value: TWorkEvent);
    procedure SetOnWorkBeginEvent(const Value: TWorkBeginEvent);
    procedure SetOnWorkEndEvent(const Value: TWorkEndEvent);
    function GetAPIUrl: string;
    procedure SetBasicAuth;
    procedure SetUserAddress(const Value: string);
    procedure SetUserPassword(const Value: string);
    procedure SetUserAgent(const Value: string);
    procedure SetUseProxy(const Value: Boolean);
    function GetStrPhotoType(photo_type: TPhotoType): string;
    function GetStrLicense(license: TPhotoLicense): string;
    function GetStrPermType(p: TPermType): string;
    function GetStrPermType2(p: TPermType2): string;
    function GetErrorMessageOut: string;
  public
    FlagHalt: Boolean;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure DisconnectSocket; // Cancel Connecting...
    procedure UseProxyIESetting;
    procedure ProxyClear;
    procedure SetLogDebug(logger: TIdLogDebug);
    // api basic method
    function CheckErrorCode(xml: string): Boolean;
    function ApiCommand(command:string; data: TPMultiForm): string;

    //todo: Primitive Method
    function api_nop: string;
    function api_photo_album(user_id: Integer): string;
    function api_user_group: string;
    function api_photo_add(filename: UTF8String; album_id: Integer;
      photo_title, tag, comment: UTF8String; date:TDateTime = 0): UTF8String;
    function api_photo_add_tag(user_id, photo_id: Integer; tag: string): string;
    function api_photo_comment(user_id: Integer): string;
    function api_photo_list(photo_type: string; user_id, album_id: Integer; license: string): string;
    function api_photo_add_album(name, description: string; perm_type:TPermType;
      perm_type2: TPermType2; perm_id: Integer): string;
    //todo: Wrap Method
    function CheckAuth: Boolean; // OCł`FbN
    function GetPhotoAlbumList(user_id: Integer = 0): string; // Aoꗗ擾
    function GetUserGroupList: string; // [U[O[vꗗ擾
    function UploadPhoto(filename: UTF8String; album_id: Integer; photo_title: UTF8String;
      tag:UTF8String=''; comment:UTF8String=''; date: TDateTime = 0): Boolean; // ʐ^Abv[h
    function PhotoAddTag(user_id, photo_id: Integer; tag: string): Boolean; // ʐ^Ƀ^O𑫂
    function GetPhotoComment(user_id: Integer): string; // ʐ^ɃRg
    function GetPhotoList(photo_type: TPhotoType; user_id, album_id: Integer;
      license: TPhotoLicense): string; // ʐ^̈ꗗ擾

    // Property
    property UserAddress: string read FUserAddress write SetUserAddress;
    property UserPassword: string read FUserPassword write SetUserPassword;
    property UserAgent: string read FUserAgent write SetUserAgent;
    property APIUrl: string read GetAPIUrl write FAPIUrl;
    property ErrorMessage: string read FErrorMessage;
    property ErrorMessageOut: string read GetErrorMessageOut;
    property UseProxy: Boolean read FUseProxy write SetUseProxy;
    property user_id: Integer read FUserId;
    property photo_id: Integer read FPhotoId;

    // Event
    property OnWork: TWorkEvent read FOnWork write SetOnWork;
    property OnWorkBegin: TWorkBeginEvent read FOnWorkBegin write SetOnWorkBeginEvent;
    property OnWorkEnd: TWorkEndEvent read FOnWorkEnd write SetOnWorkEndEvent;
  end;


function xml2csv(xml, rootname, items: string): string; // items = "id,name,num..."
procedure savetofile(fname, value: string);
function encodetag(text: string): string;

implementation

uses IdHTTPHeaderInfo, strunit, jconvert, jconvertex, csvDB, Registry,
  unit_i18n;

function encodetag(text: string): string;
begin
  text := JReplace(text, '&', '&amp;', True);
  text := JReplace(text, '<', '&lt;', True);
  text := JReplace(text, '>', '&amp;', True);
  Result := text;
end;

procedure debug(s:string);
begin
  Windows.MessageBox(0, PChar(s), 'debug', MB_OK);
end;

function xml2csv(xml, rootname, items: string): string; // items = "id,name,num..."
var
  sitems: TStringList;
  tags, tag, itemname, s: string;
  i: Integer;
begin
  Result := '';
  sitems := TStringList.Create;
  try
    sitems.Text := JReplace(items, ',', #13#10, True);
    tags := GetTags(xml, rootname);
    while tags <> '' do
    begin
      tag := GetTag(tags, rootname);
      for i := 0 to sitems.Count - 1 do
      begin
        itemname := Trim(sitems.Strings[i]);
        if itemname = '' then
        begin
          Result := Result + ',';
          Continue;
        end;
        s := Trim(GetTag(tag, itemname));
        if s = '' then break;
        // DELETE CDATA
        s := JReplace(s, '<![CDATA[', '', False);
        s := DeleteTag(Trim(JReplace(s, ']]>', '', False)));
        s := DeleteTag(s);
        s := JReplace(s, '&lt;', '<', True);
        s := JReplace(s, '&gt;', '>', True);
        s := JReplace(s, '&amp;', '&', True);
        s := JReplace(s, ',', 'C', True);
        //
        Result := Result + s + ',';
      end;
      Result := Result + #13#10;
    end;
  finally
    sitems.Free;
  end;
end;

procedure savetofile(fname, value: string);
var
  m: TMemoryStream;
begin
  m := TMemoryStream.Create;
  try
    m.Write(value[1], Length(value));
    m.SaveToFile(fname);
  finally
    m.Free;
  end;
end;

{ TPhotozouApi }

function TPhotozouApi.ApiCommand(command: string;
  data: TPMultiForm): string;
var
  res, dat: TStringStream;
  i: Integer;
begin
  FErrorMessage := '';
  
  if (timeGetTime - FLastTime) < 1000 then
  begin
    for i := 0 to 10 do
    begin
      if FlagHalt then
      begin
        Result := '';
        Exit;
      end;
      Application.ProcessMessages;
      sleep(100);
    end;
  end;
  res      := TStringStream.Create('');
  dat      := TStringStream.Create('');
  try
    try
      SetBasicAuth;
      if data = nil then
      begin
        // get
        FHttp.Intercept := FLogger;
        Result := FHttp.Get(APIUrl + command);
      end else
      begin
        // post
        dat.WriteString(data.GetPostData);
        FHttp.Request.ContentType := data.GetContetType;
        FHttp.Post(
          APIUrl + command,
          dat, res
        );
        //debug
        //savetofile('sent.txt', dat.DataString);
        Result := res.DataString;
      end;
      Result := Utf8Tosjis( Result );
    except
      on e: Exception do
      begin
        FErrorMessage := FErrorMessage + e.Message + #13#10;
      end;
    end;
    FLastTime := timeGetTime;
  finally
    res.Free;
    dat.Free;
  end;
end;

function TPhotozouApi.api_nop: string;
begin
  Result := ApiCommand(API_COMMAND_NOP, nil);
end;

function TPhotozouApi.api_photo_add(filename: UTF8String; album_id: Integer;
      photo_title, tag, comment: UTF8String; date:TDateTime): UTF8String;
var
  m: TMemoryStream;
  p: TPMultiForm;
  s: string;
begin
  p := TPMultiForm.Create;
  try
    m := TMemoryStream.Create;
    try
      m.LoadFromFile(filename);
      SetLength(s, m.Size);
      m.Position := 0;
      m.Read(s[1], m.Size);
    finally
      m.Free;
    end;

    // set
    p.AddItem('dummy', UTF8Encode(WideString(''))); // ΍
    p.AddFileItem('photo', filename, s);
    p.AddItem('album_id', IntToStr(album_id));
    p.AddItem('photo_title', photo_title);
    if date <> 0 then
    begin
      p.AddItem('date_type', 'date');
      p.AddItem('year', FormatDateTime('yyyy',date));
      p.AddItem('month',  FormatDateTime('m',date));
      p.AddItem('day', FormatDateTime('d',date));
    end;
    if tag <> '' then
    begin
      p.AddItem('tag',tag);
    end;
    if comment <> '' then
    begin
      p.AddItem('comment', comment);
    end;

    // post
    try
      Result := ApiCommand(API_COMMAND_PHOTO_ADD, p);
    except
      raise;
    end;
  finally
    p.Free;
  end;
end;

function TPhotozouApi.api_photo_add_tag(user_id, photo_id: Integer;
  tag: string): string;
var
  postdata : TPMultiForm;
begin
  postdata := TPMultiForm.Create;
  try
    // add item
    postdata.AddItem('user_id', IntToStr(user_id));
    postdata.AddItem('photo_id',IntToStr(photo_id));
    postdata.AddItem('tag',     tag);
    // post
    Result := ApiCommand(API_COMMAND_PHOTO_ADD_TAG, postdata);
  finally
    postdata.Free;
  end;
end;

function TPhotozouApi.api_photo_album(user_id: Integer): string;
var
  postdata : TPMultiForm;
begin
  postdata := TPMultiForm.Create;
  try
    // add item
    postdata.AddItem('user_id',IntToStr(user_id));
    // post
    Result := ApiCommand(API_COMMAND_PHOTO_ALBUM, postdata);
  finally
    postdata.Free;
  end;
end;

function TPhotozouApi.api_photo_comment(user_id: Integer): string;
var
  postdata : TPMultiForm;
begin
  postdata := TPMultiForm.Create;
  try
    // add item
    postdata.AddItem('user_id',IntToStr(user_id));
    // post
    Result := ApiCommand(API_COMMAND_PHOTO_COMMENT, postdata);
  finally
    postdata.Free;
  end;
end;

function TPhotozouApi.api_user_group: string;
begin
  Result := ApiCommand(API_COMMAND_USER_GROUP, nil);
end;

function TPhotozouApi.CheckAuth: Boolean;
begin
  Result := CheckErrorCode(api_nop);
end;

function TPhotozouApi.CheckErrorCode(xml: string): Boolean;
var
  code: string;
begin
  code    := LowerCase(Trim(GetTagAttribute(xml, 'rsp','stat')));
  Result  := (code = 'ok');
  if Result = False then
  begin
    FErrorMessage := Trim(FErrorMessage + GetTagAttribute(xml, 'err', 'msg', True) + #13#10);
  end;
end;

constructor TPhotozouApi.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  // Create
  FHttp := TIdHTTP.Create(Self);
  FHttp.ReadTimeout := 10 * 1000;
  // init
  FAPIUrl    := 'http://api.photozou.jp/rest/';
  FErrorMessage := '';
  FLastTime  := 0;
  FlagHalt   := False;
  FUserAgent := Application.Name + '(DelphiAPI)';
end;

destructor TPhotozouApi.Destroy;
begin
  FreeAndNil(FHttp);
  inherited;
end;

procedure TPhotozouApi.DisconnectSocket;
begin
  FlagHalt := True;
  try
    FErrorMessage := Trim(FErrorMessage + 'CLOSED GRACEFULLY') + #13#10;
    FHttp.Disconnect;
    FHttp.DisconnectSocket;
  except
    try
      FHttp.Socket.Close;
    except
    end;
  end;
end;

function TPhotozouApi.GetAPIUrl: string;
begin
  Result := FAPIUrl;
  if Copy(Result, Length(Result), 1) <> '/' then
  begin
    Result := Result + '/';
  end;
end;

function TPhotozouApi.GetPhotoAlbumList(user_id: Integer): string;
var
  code, res, username, s: string;
  csv, users: TCsvDb;
  i, j, perm_id: Integer;
begin
  Result := '';
  try
    res  := api_photo_album(user_id);

    code := Trim(GetTagAttribute(res, 'rsp', 'stat'));
    if code <> 'ok' then // error
    begin
      res := GetTags(res, 'err');
      FErrorMessage := GetTagAttribute(res, 'err', 'msg',True);
      Exit;
    end;
    //
    s := res;
    s := GetTag(s, 'rsp');
    s := GetTag(s, 'user_id');
    s := Trim(DeleteTag(s));
    FUserId := StrToIntDef(s, 0);
    //
    users := TCsvDb.Create;
    csv := TCsvDb.Create;
    try
      try
        users.Text := GetUserGroupList;
      except
      end;
      csv.Text := xml2csv(res, 'album', 'album_id,name,perm_type,perm_type2,perm_id,user_id');
      for i := 0 to csv.Count - 1 do
      begin
        username := '';
        perm_id := StrToIntDef(csv.Cells[4,i], 0);
        if perm_id > 0 then
        begin
          j := users.Find(IntToStr(perm_id), 0, 0);
          if j >= 0 then
          begin
            username := users.Cells[1, j];
          end;
        end;
        if csv.Cells[2,i] = 'allow'     then s := GetMsg('ALBUM_OPEN_TO', 'Open to %s') else s := GetMsg('ALBUM_DONT_SHOW_TO','Don''t show to %s');
        if csv.Cells[3,i] = 'net'       then s := Format(s, [GetMsg('ALBUM_NET','the Internet')]) else
        if csv.Cells[3,i] = 'everyone'  then s := Format(s, [GetMsg('ALBUM_ALL_USERS','All Users')]) else
        if csv.Cells[3,i] = 'all'       then s := Format(s, [GetMsg('ALBUM_ALL_FRIENDS','Friends from List')]) else
        if csv.Cells[3,i] = 'user_group'then s := Format(GetMsg('ALBUM_GROUP', '%s group'), [Format(s, [username])]);
        if csv.Cells[0,i] = '' then Continue;
        Result := Result + csv.Cells[0, i] + ',' + csv.Cells[1,i] + '(' + s + ')' + #13#10;
      end;
    finally
      csv.Free;
    end;
  except
  end;
end;

function TPhotozouApi.GetPhotoComment(user_id: Integer): string;
begin
  Result := api_photo_comment(user_id);
end;

function TPhotozouApi.GetUserGroupList: string;
var
  ret: string;
begin
  ret := api_user_group;
  Result := xml2csv(ret, 'user_group','group_id,name');
end;

function TPhotozouApi.PhotoAddTag(user_id, photo_id: Integer;
  tag: string): Boolean;
begin
  Result := CheckErrorCode(api_photo_add_tag(user_id, photo_id, tag));
end;


const
  proxy_key = 'Software\Microsoft\Windows\CurrentVersion\Internet Settings';

procedure TPhotozouApi.UseProxyIESetting;
var
  ProxyServer: string;
  ProxyPort: Integer;
  reg: TRegistry;
  UseProxy: DWORD;
  FProxy, FPort: string;

  procedure getHttpProxy(var Xhost, Xport: string);
  var
    s, key, value: string;
  begin
    // setting
    Xhost := '';
    Xport := '';
    // proxy
    s := FProxy;
    while (s <> '') do
    begin
      value := Trim(getToken(';', s));
      key   := Trim(UpperCase(getToken('=', value)));
      if value = '' then begin value := key; key := 'HTTP'; end;
      if (key = 'HTTP') then
      begin
        Xhost := Trim(getToken(':',value));
        Xport := Trim(value);
        Break;
      end;
    end;
  end;

begin
  // WXgProxyݒǂ
  // HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Internet Settings\ProxyServer
  // `
  // ftp=127.0.0.1:8000;http=127.0.0.1:8000
  reg := TRegistry.Create;
  try
    reg.RootKey := HKEY_CURRENT_USER;
    if reg.OpenKeyReadOnly(proxy_key) then
    begin
      UseProxy := reg.ReadInteger('ProxyEnable');
      if UseProxy <> 0 then
      begin
        FProxy := reg.ReadString('ProxyServer');
      end else
      begin
        FProxy := '';
      end;
      //
      getHttpProxy(ProxyServer, FPort);
      if FPort = '' then
      begin
        FPort := '8080';
      end;
      // PORT
      if UseProxy <> 0 then
      begin
        ProxyPort := StrToIntDef(FPort, 8080);
        FHttp.ProxyParams.ProxyServer         := ProxyServer;
        FHttp.ProxyParams.ProxyPort           := ProxyPort;
        FHttp.ProxyParams.BasicAuthentication := True;
        FHttp.ProxyParams.ProxyUsername       := FUserAddress;
        FHttp.ProxyParams.ProxyPassword       := FUserPassword;
        FUseProxy := True;
      end else
      begin
        FUseProxy := False;
      end;
    end;
    SetBasicAuth;
  finally
    reg.Free;
  end;
end;

procedure TPhotozouApi.SetBasicAuth;
begin
  with FHttp do
  begin
    // BASICF
    if Request.Authentication <> nil then
    begin
      Request.Authentication := nil;
    end;
    Request.BasicAuthentication := True;
    Request.Username  := FUserAddress;
    Request.Password  := FUSerPassword;
    Request.UserAgent := FUserAgent;
  end;
end;

procedure TPhotozouApi.SetOnWork(const Value: TWorkEvent);
begin
  FOnWork := Value;
  FHttp.OnWork := FOnWork;
end;

procedure TPhotozouApi.SetOnWorkBeginEvent(const Value: TWorkBeginEvent);
begin
  FOnWorkBegin := Value;
  FHttp.OnWorkBegin := FOnWorkBegin;
end;

procedure TPhotozouApi.SetOnWorkEndEvent(const Value: TWorkEndEvent);
begin
  FOnWorkEnd := Value;
  FHttp.OnWorkEnd := FOnWorkEnd;
end;

procedure TPhotozouApi.SetUserAddress(const Value: string);
begin
  FUserAddress := Value;
  SetBasicAuth;
end;

procedure TPhotozouApi.SetUserPassword(const Value: string);
begin
  FUserPassword := Value;
  SetBasicAuth;
end;

function TPhotozouApi.UploadPhoto(filename: UTF8String; album_id: Integer; photo_title: UTF8String;
      tag:UTF8String=''; comment:UTF8String=''; date: TDateTime = 0): Boolean;
var
  res: string;
  s: string;
begin
  try
    res := api_photo_add(filename, album_id, photo_title, tag, comment, date);
    Result := CheckErrorCode(res);
    //---
    if Result then
    begin
      s := res;
      s := GetTag(s, 'rsp');
      s := GetTag(s, 'photo_id');
      s := Trim(DeleteTag(s));
      FPhotoId := StrToIntDef(s ,0);
    end;
  except on e: Exception do
    raise;
  end;
end;

procedure TPhotozouApi.ProxyClear;
begin
  FHttp.ProxyParams.Clear;
end;

procedure TPhotozouApi.SetUserAgent(const Value: string);
begin
  SetBasicAuth;
  FUserAgent := Value;
end;

procedure TPhotozouApi.SetUseProxy(const Value: Boolean);
begin
  FUseProxy := Value;
  if FUseProxy then
  begin
    UseProxyIESetting;
  end else
  begin
    ProxyClear;
  end;
end;

procedure TPhotozouApi.SetLogDebug(logger: TIdLogDebug);
begin
  FLogger := logger;
end;

function TPhotozouApi.api_photo_list(photo_type: string; user_id,
  album_id: Integer; license: string): string;
var
  postdata : TPMultiForm;
  s: string;
begin
  postdata := TPMultiForm.Create;
  try
    // add item
    postdata.AddItem('type', photo_type);

    if user_id <> 0 then s := IntToStr(user_id) else s := '';
    postdata.AddItem('user_id',   s);

    if album_id <> 0 then s := IntToStr(album_id) else s := '';
    postdata.AddItem('album_id',  s);

    postdata.AddItem('license',   license);

    // post
    Result := ApiCommand(API_COMMAND_PHOTO_LIST, postdata);
  finally
    postdata.Free;
  end;
end;

function TPhotozouApi.GetPhotoList(photo_type: TPhotoType; user_id,
  album_id: Integer; license: TPhotoLicense): string;
var
  res: Boolean;
  xml: string;
begin
  try
    xml := api_photo_list(
      GetStrPhotoType(photo_type),
      user_id,
      album_id,
      GetStrLicense(license)
    );
    res := CheckErrorCode(xml);
  except
    res := False;
  end;
  if not res then Exit;
  Result := xml2csv(xml, 'photo', 'photo_id,user_id,album_id,pic_title,date');
end;


function TPhotozouApi.GetStrLicense(license: TPhotoLicense): string;
begin
  Result := '';
  case license of
    by      : Result := 'by';
    by_sa   : Result := 'by_sa';
    by_nd   : Result := 'by_nd';
    by_nc   : Result := 'by_nc';
    by_nc_sa: Result := 'by_nc_sa';
    by_nc_nd: Result := 'by_nc_nd';
  end;
end;

function TPhotozouApi.GetStrPhotoType(photo_type: TPhotoType): string;
begin
  Result := '';
  case photo_type of
    ptPublic  : Result := 'public';
    ptEveryone: Result := 'everyone';
    ptAlbum   : Result := 'album';
    ptComment : Result := 'comment';
    ptFriend  : Result := 'friend';
    ptPopular : Result := 'popular';
    ptCC      : Result := 'cc';
  end;
end;

function TPhotozouApi.api_photo_add_album(name, description: string;
  perm_type:TPermType; perm_type2: TPermType2; perm_id: Integer): string;
var
  postdata : TPMultiForm;
begin
  postdata := TPMultiForm.Create;
  try
    // add item
    postdata.AddItem('name', name);
    postdata.AddItem('description', description);
    postdata.AddItem('perm_type',   GetStrPermType(perm_type));
    postdata.AddItem('perm_type2',  GetStrPermType2(perm_type2));
    postdata.AddItem('perm_id',     IntToStr(perm_id));
    // post
    Result := ApiCommand(API_COMMAND_PHOTO_ADD_ALBUM, postdata);
  finally
    postdata.Free;
  end;
end;

function TPhotozouApi.GetStrPermType(p: TPermType): string;
begin
  case p of
    allow:  Result := 'allow';
    deny:   Result := 'deny';
  end;
end;

function TPhotozouApi.GetStrPermType2(p: TPermType2): string;
begin
  case p of
    net         : Result := 'net';
    everyone    : Result := 'everyone';
    all         :  Result := 'all';
    user_group  : Result := 'user_group';
  end;
end;

function TPhotozouApi.GetErrorMessageOut: string;
var
  s: TStringList;
begin
  s := TStringList.Create;
  try
    s.Text := FErrorMessage;
    Result := s.Values['MSG'];
    if Result = '' then
    begin
      Result := FErrorMessage;
    end;
  finally
    s.Free;
  end;
end;

{ TPMultiForm }

procedure TPMultiForm.AddFileItem(key, filename, val: UTF8String; ctype: UTF8String = '');
var
  p: PPMultiFormItem;
  ext: UTF8String;
begin
  //Content-Disposition: form-data; name="photo"; filename="C:\Documents and Settings\kujira\My Documents\My Pictures\ss\32-32.gif"
  //Content-Type: image/gif
  ext := LowerCase(ExtractFileExt(filename));

  if ctype = '' then
  begin
    if ext = '.gif'  then ctype := 'image/gif'   else
    if ext = '.jpg'  then ctype := 'image/jpeg'  else
    if ext = '.jpeg' then ctype := 'image/jpeg'  else
    if ext = '.png'  then ctype := 'image/png'   else
    if ext = '.3gp'  then ctype := 'video/3gpp'  else
    if ext = '.mpeg' then ctype := 'video/mpeg'  else
    if ext = '.mpg'  then ctype := 'video/mpeg'  else
    if ext = '.avi'  then ctype := 'video/avi'   else
    if ext = '.asf'  then ctype := 'video/asf'   else
    if ext = '.wmv'  then ctype := 'video/wmv'   else
    if ext = '.mov'  then ctype := 'video/mov'   else
    if ext = '.flv'  then ctype := 'video/x-flv' else
    if ext = '.mp4'  then ctype := 'video/mp4'   else
    ctype := 'application/octet-stream';
  end;

  New(p);
  p^.key := 'Content-Disposition: form-data; name="'+key+'"; filename="'+filename+'"'#13#10 +
            'Content-Type: ' + ctype;
  p^.val := val;
  Self.Add(p);
end;

procedure TPMultiForm.AddItem(key, val: UTF8String);
var
  p: PPMultiFormItem;
begin
  New(p);
  p^.key := 'Content-Disposition: form-data; name="'+key+'"';
  p^.val := val;
  Self.Add(p);
end;

procedure TPMultiForm.Clear;
var
  i: Integer;
  p: PPMultiFormItem;
begin
  for i := 0 to Count - 1 do
  begin
    p := Items[i];
    Dispose(p);
    Items[i] := nil;
  end;
  inherited Clear;
end;

function TPMultiForm.getBoundary: UTF8String;
begin
  if FBoundary = '' then
  begin
    FBoundary := '---------------------------7d617731130548';
  end;
  Result := FBoundary;
end;

function TPMultiForm.GetContetType: UTF8String;
begin
  // Check Boundary
  if Boundary = '' then Boundary := '---------------------------7d617731130548';

  Result := 'multipart/form-data; boundary=' + Boundary;
end;

function TPMultiForm.GetPostData: UTF8String;
var
  i: Integer;
  p: PPMultiFormItem;
begin
  Result := '';

  for i := 0 to Count - 1 do
  begin
    p := Items[i];
    Result := Result + '--' + Boundary + #13#10;
    Result := Result + p^.key + #13#10;
    Result := Result + '' + #13#10;
    Result := Result + p^.val + #13#10;
  end;

  Result := Result + '--' + Boundary + '--';
end;

end.
